function findPostPath(element) {
    while (element && element.getAttribute('data-testid') !== 'tweet') {
        element = element.parentNode;
    }
    if (!element) return null;  // If no tweet element found
    const timeElement = element.querySelector('time');
    return timeElement ? timeElement.parentNode.getAttribute('href') : null;
}

function isElementImage(element) {
    // Look at its children and see if any are img tags
    const children = element.children;
    for (let i = 0; i < children.length; i++) {
        if (children[i].tagName === 'IMG') {
            return true;
        }
    }
    return false;
}

function onVideoClick(event, element) {
    if (isElementImage(element)) return;

    let parent = element.parentNode;
    let grandparent = parent.parentNode;
    let mediaContainer = grandparent.parentNode;

    let videoElements = Array.from(
        mediaContainer.querySelectorAll('[data-testid="tweetPhoto"]')
    ).filter((elem) => elem.getAttribute('aria-label') !== 'Image');

    let videoIndex = videoElements.indexOf(element);
    let postPath = findPostPath(mediaContainer);
    if (!postPath) {
        console.log('Could not find post path');
    }

    window.JSInterface.onVideoClick(videoIndex + 1, postPath);
}

const listenerMap = new Map();

function addClickListeners() {
    let elements = document.querySelectorAll('[data-testid="tweetPhoto"]');
    elements.forEach((element) => {
        if (listenerMap.has(element)) return;
        const listener = (e) => onVideoClick(e, element);
        element.addEventListener('click', listener);
        listenerMap.set(element, listener);
    });
}

setTimeout(addClickListeners, 1000);
const observer = new MutationObserver(function(mutationsList, observer) {
    if (mutationsList.some(mutation => mutation.type === 'childList')) {
        addClickListeners();
    }
});
observer.observe(document, { childList: true, subtree: true });